import React, { Children, Fragment } from "react";
import { Grid } from "@components/Grid";
import { Link } from "@components/Link";
import { ConditionalWrapper } from "./ConditionalWrapper";

type Image = {
  src: string;
  alt: string;
  href?: string;
};

type BaseProps = {
  title?: string;
  columns: 2 | 3 | 4 | 5 | 6;
  className?: string;
  customGap?: string;
  customResponsiveColumn?: string;
  newTab?: boolean;
};

type EntriesOrChildren =
  | {
      children: React.ReactNode;
      entries?: never;
    }
  | {
      children?: never;
      entries: Image[];
    };

export type LogoGridProps = BaseProps & EntriesOrChildren;

export function LogoGrid({
  title,
  columns,
  className = "",
  customGap = "",
  customResponsiveColumn = "",
  children,
  entries,
  newTab = true,
}: LogoGridProps) {
  const GAP_MAPPING: Record<string, string> = {
    2: "fs-gap-10",
    3: "fs-gap-10",
    4: "fs-gap-x-10 fs-gap-y-10 sm:fs-gap-x-12 sm:fs-gap-y-12 xl:fs-gap-x-10 xl:fs-gap-y-10",
    5: "fs-gap-x-16 fs-gap-y-16 sm:fs-gap-x-16 sm:fs-gap-y-12 xl:fs-gap-x-1 xl:fs-gap-y-10",
    6: "fs-gap-x-1 fs-gap-y-14 sm:fs-gap-x-16 sm:fs-gap-y-12 xl:fs-gap-x-1 xl:fs-gap-y-14",
  };
  const RESPONSIVE_COLUMN_MAPPING: Record<string, string> = {
    2: "",
    3: "fs-grid-cols-2",
    4: "fs-grid-cols-2 lg:fs-grid-cols-3",
    5: "fs-grid-cols-2 sm:fs-grid-cols-4",
    6: "fs-grid-cols-4",
  };

  return (
    <div className={className}>
      {title && (
        <h5 className="fs-mb-14 fs-font-plex fs-text-base fs-text-center fs-leading-6 fs-font-semibold fs-tracking-wider fs-text-grey-900 fs-uppercase">
          {title}
        </h5>
      )}
      <Grid
        columns={columns}
        customGap={customGap ? customGap : GAP_MAPPING[columns]}
        customResponsiveColumn={
          customResponsiveColumn
            ? customResponsiveColumn
            : RESPONSIVE_COLUMN_MAPPING[columns]
        }
      >
        {children && Children.map(children, (child) => child)}
        {entries &&
          entries.map((entry, i) => (
            <Fragment key={i}>
              {entry.href ? (
                <Link
                  to={entry.href}
                  {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
                  className={`fs-flex fs-justify-center fs-items-center`}
                >
                  <Logo src={entry.src} alt={entry.alt} />
                </Link>
              ) : (
                <div className={`fs-flex fs-justify-center fs-items-center`}>
                  <Logo src={entry.src} alt={entry.alt} />
                </div>
              )}
            </Fragment>
          ))}
      </Grid>
    </div>
  );
}

export interface LogoItemProps {
  src: string;
  alt: string;
  href?: string;
}

export function Logo({ src, alt, href }: LogoItemProps) {
  const classes = "fs-size-full fs-object-contain";
  return (
    <ConditionalWrapper
      condition={Boolean(href)}
      wrapper={(children) => <Link to={href}>{children}</Link>}
    >
      <img className={classes} src={src} alt={alt} />
    </ConditionalWrapper>
  );
}
