import React, { Fragment, useState, useEffect } from "react";
import { Search as SearchIcon } from "@carbon/icons-react";
import clsx from "clsx";

type SearchProps = {
  shouldDoTransition: boolean;
  onClose?: () => void;
  onSearch?: (keyword: string) => void;
};

export interface Props {
  faded?: boolean;
  onSearch: (keyword: string) => void;
  onHover?: (value: boolean) => void;
}

export function Search({ onSearch, onHover, faded }: Props) {
  const [isSearchOpen, setIsSearchOpen] = useState(false);
  const [shouldDoTransition, setShouldDoTransition] = useState(false);
  const classes =
    "fs-cursor-pointer fs-transition-all fs-ease-in fs-duration-100 hover:fs-text-grey-900";

  useEffect(() => {
    if (isSearchOpen) {
      setTimeout(() => {
        setShouldDoTransition(true);
      }, 75);
    }
  }, [isSearchOpen]);

  useEffect(() => {
    if (!shouldDoTransition) {
      setTimeout(() => {
        setIsSearchOpen(false);
      }, 75);
    }
  }, [shouldDoTransition]);

  return (
    <Fragment>
      <span
        onMouseEnter={() => onHover && onHover(true)}
        onMouseLeave={() => onHover && onHover(false)}
        onClick={() => setIsSearchOpen(true)}
        className={clsx(
          classes,
          faded ? "fs-text-grey-400" : "fs-text-grey-900",
        )}
      >
        <SearchIcon size="20" />
      </span>
      {isSearchOpen && (
        <SearchContainer
          shouldDoTransition={shouldDoTransition}
          onSearch={onSearch}
          onClose={() => setShouldDoTransition(false)}
        />
      )}
    </Fragment>
  );
}

function SearchContainer({
  shouldDoTransition,
  onClose,
  onSearch,
}: SearchProps) {
  return (
    <div className="fs-fixed fs-inset-0 fs-z-50 fs-flex fs-items-start fs-justify-center fs-pt-16 sm:fs-pt-24">
      <Backdrop
        shouldDoTransition={shouldDoTransition}
        onClose={() => onClose && onClose()}
      />
      <Input onSearch={onSearch} shouldDoTransition={shouldDoTransition} />
    </div>
  );
}

function Input({ shouldDoTransition, onSearch }: SearchProps) {
  const handleSubmit = (e: React.KeyboardEvent<HTMLInputElement>) => {
    if (e.key === "Enter") {
      onSearch && onSearch(e.currentTarget.value);
    }
  };
  return (
    <div
      className={clsx(
        "fs-relative fs-w-full fs-max-w-lg fs-transform fs-px-4 fs-transition-all fs-opacity-0 fs-scale-0",
        shouldDoTransition && "fs-opacity-100 !fs-scale-100",
      )}
    >
      <div className="fs-overflow-hidden fs-rounded-lg fs-bg-white fs-shadow-md">
        <div className="fs-relative">
          <input
            onKeyDown={handleSubmit}
            className="fs-block fs-w-full fs-appearance-none fs-bg-transparent fs-py-4 fs-pl-4 fs-pr-12 fs-text-base fs-text-slate-900 placeholder:fs-text-slate-600 focus:fs-outline-none sm:fs-text-sm sm:fs-leading-6"
            placeholder="Find anything..."
          />
          <SearchIcon
            size="20"
            className="fs-absolute fs-right-3.5 fs-top-1/2 -fs-translate-y-2/4"
          />
        </div>
      </div>
    </div>
  );
}

function Backdrop({ shouldDoTransition, onClose }: SearchProps) {
  return (
    <div
      onClick={onClose}
      className={clsx(
        "fs-fixed fs-inset-0 fs-bg-slate-900/25 fs-backdrop-blur fs-transition-opacity fs-opacity-0",
        shouldDoTransition && "fs-opacity-100",
      )}
    />
  );
}
