import { useEffect, useState, Children, isValidElement } from "react";
import type { PropsWithChildren } from "react";
import { Button } from "@components/Button";
import { CommonUtils } from "@utils/CommonUtils";
import { Link } from "@components/Link";
import clsx from "clsx";

export interface TableOfContentsProps {
  ctaOnClick?: () => unknown;
  ctaCaption?: string;
}

interface IHeadlineProps {
  id: string;
  label: string;
  target: string;
}

export function TableOfContents({
  children,
  ctaOnClick,
  ctaCaption,
}: PropsWithChildren<TableOfContentsProps>) {
  const [activeHeadline, setActiveHeadline] = useState<string | null>(null);
  const [headlineData, setHeadlineData] = useState<IHeadlineProps[]>([]);

  useEffect(() => {
    const data: IHeadlineProps[] = [];
    Children.forEach(children, (child) => {
      if (!isValidElement(child)) return;
      data.push({
        id: CommonUtils.generateID(),
        label: child.props.children,
        target: child.props.target,
      });
    });
    setHeadlineData(data);
    setActiveHeadline(data[0].id);
  }, []);

  return (
    <div className="fs-flex">
      <div className="fs-w-full">
        <ul className="fs-font-inter fs-mb-5">
          {headlineData?.map((headline) => (
            <TocItem
              label={headline.label}
              key={headline.id}
              className="block"
              target={headline.target}
              isActive={activeHeadline === headline.id}
              onClick={() => {
                setActiveHeadline(headline.id);
              }}
            />
          ))}
        </ul>
        {ctaCaption && (
          <Button
            text={ctaCaption}
            type="primary"
            size="lg"
            onClick={ctaOnClick}
          />
        )}
      </div>
    </div>
  );
}

export interface TocItemProps {
  label: string;
  onClick: () => void;
  isActive: boolean;
  className?: string;
  target: string;
}

export function TocItem({
  label,
  target,
  onClick,
  isActive,
  className = "",
}: TocItemProps) {
  return (
    <li className="fs-group fs-mb-3">
      <Link onClick={onClick} className={className} to={target}>
        <span
          className={clsx(
            "fs-relative fs-overflow-hidden fs-text-sm fs-leading-5 fs-font-medium fs-text-black fs-multi-line-grey-900 group-hover:fs-multi-line-underline fs-cursor-pointer",
            isActive && "fs-multi-line-underline",
          )}
        >
          {label}
        </span>
      </Link>
    </li>
  );
}

export const Headline: React.FC<{
  target: string;
  children: React.ReactNode;
}> = () => {
  return null;
};
