import { CSSProperties } from "react";
import { INewsCard } from "@interfaces/INews";
import { Link } from "@components/Link";
import { Pill } from "@components/Pill";
import { Time, Location, Launch } from "@carbon/icons-react";
import clsx from "clsx";
import { CommonUtils } from "@utils/CommonUtils";

type CardColorVariant = "dark" | "light";
type PillColorType = "dark" | "white";

export function VerticalNewsCard({
  image,
  title,
  description,
  date,
  location,
  category,
  href,
  variant,
  className = "",
  customBackgroundColor,
  textColor = "dark",
  externalLinkIndicator,
  newTab,
  id,
}: INewsCard) {
  const inlineStyle: CSSProperties = {
    backgroundColor: customBackgroundColor,
  };

  const PILL_COLOR_MAPPING: Record<CardColorVariant, PillColorType> = {
    dark: "white",
    light: "dark",
  };

  const STYLE_MAPPING: Record<string, Record<string, string>> = {
    base: {
      outerContainer:
        "fs-snap-start fs-w-[280px] sm:fs-w-[380px] md:fs-w-[420px] xl:fs-w-[460px] fs-shrink-0",
    },
    dark: {
      outerContainer: "fs-bg-grey-900",
      contentContainer:
        "fs-group fs-aspect-[56/51] sm:fs-aspect-[100/61] fs-relative fs-p-6 sm:fs-p-7 md:fs-p-8",
    },
    light: {
      outerContainer: "fs-bg-pearl-500",
      contentContainer:
        "fs-group fs-aspect-[56/51] sm:fs-aspect-[100/61] fs-relative fs-p-6 sm:fs-p-7 md:fs-p-8",
    },
    custom: {
      outerContainer: "",
      contentContainer:
        "fs-group fs-aspect-[56/51] sm:fs-aspect-[100/61] fs-relative fs-p-6 sm:fs-p-7 md:fs-p-8",
    },
  };
  // Text exists only in light/dark – no custom text color is possible
  const TEXT_STYLE_MAPPING: Record<CardColorVariant, Record<string, string>> = {
    dark: {
      heading:
        "group-peer-hover:fs-multi-line-underline group-hover:fs-multi-line-underline xl:fs-multi-line-3-white fs-font-karla fs-text-2xl xl:fs-text-3xl fs-leading-8 md:fs-leading-9 fs-font-bold fs-text-white fs-line-clamp-3 sm:fs-line-clamp-2 xl:fs-line-clamp-none",
      paragraph:
        "fs-mt-2 md:fs-mt-3 fs-font-plex fs-text-base md:fs-text-lg fs-leading-6 md:fs-leading-7 fs-font-normal fs-text-grey-100 fs-line-clamp-2 md:fs-line-clamp-2 xl:fs-line-clamp-3",
      event:
        "fs-mt-2 md:fs-mt-3 fs-font-plex fs-text-base md:fs-text-lg fs-leading-6 md:fs-leading-7 fs-font-normal fs-text-grey-50",
    },
    light: {
      heading:
        "group-peer-hover:fs-multi-line-underline group-hover:fs-multi-line-underline xl:fs-multi-line-3-grey-900 fs-font-karla fs-text-2xl xl:fs-text-3xl fs-leading-8 md:fs-leading-9 fs-font-bold fs-text-grey-900 fs-line-clamp-3 sm:fs-line-clamp-2 xl:fs-line-clamp-none",
      paragraph:
        "fs-mt-2 md:fs-mt-4 fs-font-plex fs-text-base md:fs-text-lg fs-leading-6 md:fs-leading-7 fs-font-normal fs-text-grey-900 fs-line-clamp-2 md:fs-line-clamp-2 xl:fs-line-clamp-3",
      event:
        "fs-mt-2 md:fs-mt-4 fs-font-plex fs-text-base md:fs-text-lg fs-leading-6 md:fs-leading-7 fs-font-normal fs-text-grey-900",
    },
  };

  return (
    <div
      id={id}
      style={inlineStyle}
      className={clsx(
        variant
          ? STYLE_MAPPING[variant]["outerContainer"]
          : STYLE_MAPPING["custom"]["outerContainer"],
        STYLE_MAPPING["base"]["outerContainer"],
        className,
      )}
    >
      <div className="fs-peer fs-aspect-square fs-relative">
        {externalLinkIndicator && (
          <div className="fs-absolute fs-left-1/2 fs-top-1/2 -fs-translate-x-1/2 -fs-translate-y-1/2">
            <Pill
              newTab
              type={
                variant
                  ? PILL_COLOR_MAPPING[variant]
                  : PILL_COLOR_MAPPING[textColor]
              }
              size="lg"
              rounded
              href={href}
              title={externalLinkIndicator}
              icon={<Launch className="fs-size-4 fs-mr-2" />}
            />
          </div>
        )}
        <Link
          to={href}
          {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
        >
          <img
            className="fs-size-full fs-object-cover"
            alt={image.alt}
            src={image.src}
          />
        </Link>
      </div>

      <div
        className={
          variant
            ? STYLE_MAPPING[variant]["contentContainer"]
            : STYLE_MAPPING["custom"]["contentContainer"]
        }
      >
        <div className="fs-size-full fs-relative">
          <Link
            to={href}
            {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
          >
            <h5
              className={
                variant
                  ? TEXT_STYLE_MAPPING[variant]["heading"]
                  : TEXT_STYLE_MAPPING[CommonUtils.invertDarkLight(textColor)][
                      "heading"
                    ]
              }
            >
              {title}
            </h5>
            {description && (
              <p
                className={
                  variant
                    ? TEXT_STYLE_MAPPING[variant]["paragraph"]
                    : TEXT_STYLE_MAPPING[
                        CommonUtils.invertDarkLight(textColor)
                      ]["paragraph"]
                }
              >
                {description}
              </p>
            )}
            {!description && (date || location) && (
              <div
                className={
                  variant
                    ? TEXT_STYLE_MAPPING[variant]["event"]
                    : TEXT_STYLE_MAPPING[
                        CommonUtils.invertDarkLight(textColor)
                      ]["event"]
                }
              >
                {date && (
                  <span className="fs-flex fs-items-center">
                    <Time size={16} className="fs-mr-1 md:fs-mr-2" />
                    {date}
                  </span>
                )}
                {location && (
                  <span className="fs-flex fs-items-center fs-mt-1">
                    <Location size={16} className="fs-mr-1 md:fs-mr-2" />
                    {location}
                  </span>
                )}
              </div>
            )}
          </Link>
          {category && (
            <div className="fs-absolute fs-bottom-0 fs-left-0 fs-flex fs-flex-wrap">
              {category.title && (
                <Pill
                  type={
                    variant
                      ? PILL_COLOR_MAPPING[variant]
                      : PILL_COLOR_MAPPING[
                          CommonUtils.invertDarkLight(textColor)
                        ]
                  }
                  size="sm"
                  rounded
                  href={category.href}
                  title={category.title}
                />
              )}
            </div>
          )}
        </div>
      </div>
    </div>
  );
}
